@@ -30,8 +30,9 @@
 #include "pmpd_export.h"
 #include "pmpd_version.h"
 
-
 #define max(a,b) ( ((a) > (b)) ? (a) : (b) )
+#define min(a,b) ( ((a) < (b)) ? (a) : (b) )
+#define clamp(x,lo,hi) ( min(hi, max(lo, x)) )
 
 #define NB_MAX_LINK_DEFAULT 10000
 #define NB_MAX_MASS_DEFAULT 10000
@@ -50,7 +51,7 @@ struct _mass {
     t_float posY;
     t_float forceX;
     t_float forceY;
-    t_float D; // damping de vitesse
+    t_float D; // velocity damping
     t_float Doffset;
     t_int Id;
 };
@@ -105,8 +106,10 @@ typedef struct _pmpd2d_tilde {
     t_int nb_inlet, nb_outlet, nb_max_in, nb_max_out;
     t_int nb_inPosX, nb_inPosY, nb_inForceX, nb_inForceY;
     t_int nb_outPosX, nb_outPosY, nb_outSpeedX, nb_outSpeedY, nb_outSpeed;
+    t_float minX, maxX, minY, maxY;
     t_sample f; // used for signal inlet
     t_int nb_loop; // to be able not to compute everything a each iteration
+    t_int test_minmax;
 } t_pmpd2d_tilde;
 
 t_int *pmpd2d_tilde_perform(t_int *w)
@@ -115,10 +118,8 @@ t_int *pmpd2d_tilde_perform(t_int *w)
     t_pmpd2d_tilde *x = (t_pmpd2d_tilde *)(w[1]);
     t_int n = w[2]; // sample count from sp[0]->s_n
 
-    t_float F, FX, FY, L, LX, LY, deltaL, invL;
+    t_float F, FX, FY, L, LX, LY, deltaL, invL, tmpX, tmpY, speed, invSpeed;
     t_int i, si, loop;
-
-	t_float sqSpeed;
 	
     for (si=0; si<n; si++)
     {
@@ -156,13 +157,13 @@ t_int *pmpd2d_tilde_perform(t_int *w)
                 x->link[i].mass1->forceY += FY;
                 x->link[i].mass2->forceY -= FY;
             }
-
             for (i=0; i<x->nb_NLlink; i++)
             // compute forces generated by NLlinks (spring / dashpot)
             {
                 LX = x->NLlink[i].mass2->posX - x->NLlink[i].mass1->posX;
                 LY = x->NLlink[i].mass2->posY - x->NLlink[i].mass1->posY;
                 L = sqrt(LY*LY + LX*LX);
+
                 if ((L < x->NLlink[i].Lmax) && (L > x->NLlink[i].Lmin)) {
                     deltaL = L - x->NLlink[i].L0;
                     F = x->NLlink[i].K * pow(fabs(deltaL), x->NLlink[i].Pow);
@@ -184,24 +185,59 @@ t_int *pmpd2d_tilde_perform(t_int *w)
                 }
                 else x->NLlink[i].L = L; // on la sauve pour la prochaine itération (uniquement pour le "bang")
             }
-            for (i=0; i<x->nb_mass; i++)
-            {
-            // compute new masses position
-            // a mass does not move if M=0 (i.e : invM = 0)
-                if (x->mass[i].D && (sqSpeed = x->mass[i].speedX * x->mass[i].speedX + x->mass[i].speedY * x->mass[i].speedY) ) { // velocity damping
-	            	L = sqrt(sqSpeed);
-                    invL = 1 / L;
-	            	F = -(L - x->mass[i].Doffset) * x->mass[i].D;
-	                x->mass[i].forceX += F * x->mass[i].speedX * invL;
-	                x->mass[i].forceY += F * x->mass[i].speedY * invL;
-                }
-                x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
-                x->mass[i].speedY += x->mass[i].forceY * x->mass[i].invM;
-                x->mass[i].forceX = 0;
-                x->mass[i].forceY = 0;
-                x->mass[i].posX += x->mass[i].speedX;
-                x->mass[i].posY += x->mass[i].speedY;
-            }
+            if (!x->test_minmax) 
+            { // High-performance loop, but accuracy errors can occur.
+		        for (i=0; i<x->nb_mass; i++)
+		        { // compute new masses position
+		            if ( (x->mass[i].D) && (speed = x->mass[i].speedX * x->mass[i].speedX + x->mass[i].speedY * x->mass[i].speedY) ) { // velocity damping
+			            speed = sqrt(speed);
+			            invSpeed = 1/speed;
+		                F = -(speed - x->mass[i].Doffset) * x->mass[i].D;
+		                x->mass[i].forceX += F * x->mass[i].speedX * invSpeed;
+		                x->mass[i].forceY += F * x->mass[i].speedY * invSpeed;
+		            }
+		            x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
+		            x->mass[i].speedY += x->mass[i].forceY * x->mass[i].invM;
+		            x->mass[i].forceX = 0;
+		            x->mass[i].forceY = 0;
+		          
+		            x->mass[i].posX += x->mass[i].speedX;
+		            x->mass[i].posY += x->mass[i].speedY;
+		        }
+	    	}
+		    else 
+		    {  
+                for (i=0; i<x->nb_mass; i++)
+		        { // compute new masses position
+			    	if(x->mass[i].invM!=0) { // a mass does not move if M=0 (i.e : invM = 0)
+				        if ( (x->mass[i].D) && (speed = x->mass[i].speedX * x->mass[i].speedX + x->mass[i].speedY * x->mass[i].speedY) ) { // velocity damping
+					        speed = sqrt(speed);
+					        invSpeed = 1/speed;
+				            F = -(speed - x->mass[i].Doffset) * x->mass[i].D;
+				            x->mass[i].forceX += F * x->mass[i].speedX * invSpeed;
+				            x->mass[i].forceY += F * x->mass[i].speedY * invSpeed;
+				        }
+				        x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
+				        x->mass[i].speedY += x->mass[i].forceY * x->mass[i].invM;
+				        x->mass[i].forceX = 0;
+				        x->mass[i].forceY = 0;
+				      
+				        x->mass[i].posX += x->mass[i].speedX;
+				        x->mass[i].posY += x->mass[i].speedY;
+
+				        // space limitation
+				        if ( (x->mass[i].posX < x->minX) || (x->mass[i].posX > x->maxX) || (x->mass[i].posY < x->minY) || (x->mass[i].posY > x->maxY) ) 
+				        {
+				            tmpX = clamp(x->mass[i].posX, x->minX, x->maxX);
+				            tmpY = clamp(x->mass[i].posY, x->minY, x->maxY);
+				            x->mass[i].speedX -= x->mass[i].posX - tmpX;
+				            x->mass[i].speedY -= x->mass[i].posY - tmpY;
+				            x->mass[i].posX = tmpX;
+				            x->mass[i].posY = tmpY;
+				        }
+				    }
+		        }
+        	}
         }
         
         // compute output vector value
@@ -262,6 +298,7 @@ void pmpd2d_tilde_bang(t_pmpd2d_tilde *x)
     for (i=0; i<x->nb_outSpeed; i++) logpost(x, 2, "outSpeed:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeed[i].nbr_outlet, x->outSpeed[i].mass->Id, x->outSpeed[i].influence);
     for (i=0; i<x->nb_outSpeedX; i++) logpost(x, 2, "outSpeedX:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeedX[i].nbr_outlet, x->outSpeedX[i].mass->Id, x->outSpeedX[i].influence);
     for (i=0; i<x->nb_outSpeedY; i++) logpost(x, 2, "outSpeedY:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeedY[i].nbr_outlet, x->outSpeedY[i].mass->Id, x->outSpeedY[i].influence);
+    if(x->test_minmax) { logpost(x, 2, "minX:%f, maxX:%f, minY:%f, maxY:%f", x->minX, x->maxX, x->minY, x->maxY); }
 }
 
 inline int validate_index(t_pmpd2d_tilde *x, int idx, t_int count, const char* type)
@@ -414,6 +451,44 @@ void pmpd2d_tilde_setNLLCurrent(t_pmpd2d_tilde *x, t_symbol *s, int argc, t_atom
     x->NLlink[idx_NLlink].L0 += percent * (x->NLlink[idx_NLlink].L - x->NLlink[idx_NLlink].L0);
 }
 
+void pmpd2d_tilde_min(t_pmpd2d_tilde *x, t_float minX, t_float minY)
+{
+    x->minX = minX;
+    x->minY = minY;
+    x->test_minmax = 1;
+}
+
+void pmpd2d_tilde_max(t_pmpd2d_tilde *x, t_float maxX, t_float maxY)
+{
+    x->maxX = maxX;
+    x->maxY = maxY;
+    x->test_minmax = 1;
+}
+
+void pmpd2d_tilde_minX(t_pmpd2d_tilde *x, t_float min)
+{
+    x->minX = min;
+    x->test_minmax = 1;
+}
+
+void pmpd2d_tilde_maxX(t_pmpd2d_tilde *x, t_float max)
+{
+    x->maxX = max;
+    x->test_minmax = 1;
+}
+
+void pmpd2d_tilde_minY(t_pmpd2d_tilde *x, t_float min)
+{
+    x->minY = min;
+    x->test_minmax = 1;
+}
+
+void pmpd2d_tilde_maxY(t_pmpd2d_tilde *x, t_float max)
+{
+    x->maxY = max;
+    x->test_minmax = 1;
+}
+
 inline int validate_count(t_pmpd2d_tilde *x, t_int count, t_int count_max, const char* type)
 {
     if (count == count_max) {
@@ -598,6 +673,10 @@ void pmpd2d_tilde_reset(t_pmpd2d_tilde *x)
     x->nb_link      = 0;
     x->nb_NLlink    = 0;
     x->nb_mass      = 0;
+    x->minX         = -1000000;
+    x->maxX         = 1000000;
+    x->minY         = -1000000;
+    x->maxY         = 1000000;
     x->nb_inPosX    = 0;
     x->nb_inPosY    = 0;
     x->nb_inForceX  = 0;
@@ -607,6 +686,8 @@ void pmpd2d_tilde_reset(t_pmpd2d_tilde *x)
     x->nb_outSpeed  = 0;
     x->nb_outPosX   = 0;
     x->nb_outPosY   = 0;
+    x->test_minmax = 0;
+
 }
 
 void pmpd2d_tilde_free(t_pmpd2d_tilde *x)
@@ -643,14 +724,18 @@ void *pmpd2d_tilde_new(t_symbol *s, int argc, t_atom *argv)
 
     // check for flags (currently need to be positioned first)
     while (argc && argv->a_type == A_SYMBOL) {
-        if (atom_getsymbol(argv) == gensym("-m"))
+        t_symbol *flag = atom_getsymbol(argv);
+        if (flag == gensym("-m")) {
             if(g_signal_setmultiout)
                 x->multichannel = 1;
             else
                 pd_error(x, "[pmpd2d~]: no multichannel support in Pd %i.%i-%i, ignoring '-m' flag", maj, min, bug);
-        else
-            pd_error(x, "[pmpd~]: invalid argument");
-        argc--, argv++;
+            argc--, argv++;
+        }
+        else {
+            pd_error(x, "[pmpd2d~]: invalid argument %s", flag->s_name);
+            argc--, argv++;
+        }
     }
 
     x->nb_inlet = max(1, atom_getintarg(0, argc, argv));
@@ -666,9 +751,9 @@ void *pmpd2d_tilde_new(t_symbol *s, int argc, t_atom *argv)
     x->inlet_vector = (t_sample **)getbytes(x->nb_inlet * sizeof(t_sample *));
     x->outlet_vector = (t_sample **)getbytes(x->nb_outlet * sizeof(t_sample *));
 
-    x->mass      = (struct _mass *)getbytes(x->nb_max_mass * sizeof(struct _link));
+    x->mass      = (struct _mass *)getbytes(x->nb_max_mass * sizeof(struct _mass));
     x->link      = (struct _link *)getbytes(x->nb_max_link * sizeof(struct _link));
-    x->NLlink    = (struct _NLlink *)getbytes(x->nb_max_link * sizeof(struct _link));
+    x->NLlink    = (struct _NLlink *)getbytes(x->nb_max_link * sizeof(struct _NLlink));
 
     x->inPosX    = (struct _input *)getbytes(x->nb_max_in * sizeof(struct _input));
     x->inPosY    = (struct _input *)getbytes(x->nb_max_in * sizeof(struct _input));
@@ -756,6 +841,12 @@ PMPD_EXPORT void pmpd2d_tilde_setup(void)
     class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_setNLLMin, gensym("setNLLMin"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_setNLLMax, gensym("setNLLMax"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_setNLLCurrent, gensym("setNLLCurrent"), A_GIMME, 0);
+    class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_min, gensym("min"), A_DEFFLOAT, A_DEFFLOAT, 0);
+    class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_max, gensym("max"), A_DEFFLOAT, A_DEFFLOAT, 0);
+    class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_minX, gensym("minX"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_maxX, gensym("maxX"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_minY, gensym("minY"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_maxY, gensym("maxY"), A_DEFFLOAT, 0);
 
     class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_reset, gensym("reset"), 0);
     class_addmethod(pmpd2d_tilde_class, (t_method)pmpd2d_tilde_dsp, gensym("dsp"), A_CANT, 0);
