@@ -40,7 +40,8 @@ static inline t_float pmpd3d_pow(t_float x, t_float y)
 
 static inline t_float pmpd3d_mix(t_float X, t_float Y, t_float pmpd3d_mix)
 {
-    return (1-pmpd3d_mix)*X + pmpd3d_mix*Y ;
+    //return (1-pmpd3d_mix)*X + pmpd3d_mix*Y;
+    return X + (Y-X) * pmpd3d_mix;
 }
 
 t_float pmpd3d_tabread2(t_pmpd3d *x, t_float pos, t_symbol *array)
@@ -127,11 +128,11 @@ void pmpd3d_bang(t_pmpd3d *x)
 		// compute new masses position
         if (x->mass[i].mobile > 0) // only if mobile
         {
-			// amplify force that opose to movement
+			// amplify force that oppose to movement
 			if (x->mass[i].overdamp != 0)
 			{
 				tmp = x->mass[i].speedX * x->mass[i].forceX + x->mass[i].speedY * x->mass[i].forceY + x->mass[i].speedZ * x->mass[i].forceZ;
-				tmp = min(0,tmp); // overdamped only if force opose movment
+				tmp = min(0,tmp); // overdamped only if force oppose movement
 				tmp *= -x->mass[i].overdamp;
 				tmp += 1;
 				x->mass[i].forceX *= tmp;
