@@ -574,13 +574,13 @@ void pmpd3d_addL(t_pmpd3d *x, t_symbol *s, int argc, t_atom *argv)
         }
     }
     else if ( (argc == 1) && ( argv[0].a_type == A_FLOAT ) )
-    { // set a link to it's curent size
+    { // set a link to it's current size
         tmp = atom_getfloatarg(0, argc, argv);
         tmp = max(0, min( x->nb_link-1, tmp));
         x->link[tmp].L += x->link[tmp].distance;
 	}
     else if ( (argc == 1) && ( argv[0].a_type == A_SYMBOL ) )
-    { // set a class of link to there curent size
+    { // set a class of link to there current size
         for (i=0; i< x->nb_link; i++)
         {
             if ( atom_getsymbolarg(0,argc,argv) == x->link[i].Id)
@@ -634,13 +634,13 @@ void pmpd3d_setLCurrent(t_pmpd3d *x, t_symbol *s, int argc, t_atom *argv)
         }
     }
     else if ( (argc == 2) && ( argv[0].a_type == A_FLOAT ) && ( argv[1].a_type == A_FLOAT ) )
-    { // set a link to a pmpd3d_mix between it's curent length and it's size
+    { // set a link to a pmpd3d_mix between it's current length and it's size
         i = atom_getfloatarg(0, argc, argv);
         i = max(0, min( x->nb_link-1, i));
         x->link[i].L = pmpd3d_mix(x->link[i].L,x->link[i].distance,atom_getfloatarg(1, argc, argv));
     }
     else if ( (argc == 2) && ( argv[0].a_type == A_SYMBOL ) && ( argv[1].a_type == A_FLOAT ) )
-    { // set a class of link to a pmpd3d_mix between it's curent length and it's size
+    { // set a class of link to a pmpd3d_mix between it's current length and it's size
         for (i=0; i< x->nb_link; i++)
         {
             if ( atom_getsymbolarg(0,argc,argv) == x->link[i].Id)
