@@ -32,6 +32,8 @@
 
 
 #define max(a,b) ( ((a) > (b)) ? (a) : (b) )
+#define min(a,b) ( ((a) < (b)) ? (a) : (b) )
+#define clamp(x,lo,hi) ( min(hi, max(lo, x)) )
 
 #define NB_MAX_LINK_DEFAULT 10000
 #define NB_MAX_MASS_DEFAULT 10000
@@ -114,6 +116,8 @@ typedef struct _pmpd3d_tilde {
     t_int nb_outPosX, nb_outPosY, nb_outPosZ, nb_outSpeedX, nb_outSpeedY, nb_outSpeedZ, nb_outSpeed;
     t_sample f; // used for signal inlet
     t_int nb_loop; // to be able not to compute everything a each iteration
+    t_float minX, maxX, minY, maxY, minZ, maxZ;
+    t_int test_minmax;
 } t_pmpd3d_tilde;
 
 t_int *pmpd3d_tilde_perform(t_int *w)
@@ -122,7 +126,7 @@ t_int *pmpd3d_tilde_perform(t_int *w)
     t_pmpd3d_tilde *x = (t_pmpd3d_tilde *)(w[1]);
     t_int n = w[2]; // sample count from sp[0]->s_n
 
-    t_float F, FX, FY, FZ, L, LX, LY, LZ, deltaL, invL;
+    t_float F, FX, FY, FZ, L, LX, LY, LZ, deltaL, invL, tmpX, tmpY, tmpZ;
     t_int i, si, loop;
 
 	t_float sqspeed;
@@ -171,7 +175,6 @@ t_int *pmpd3d_tilde_perform(t_int *w)
                 x->link[i].mass1->forceZ += FZ;
                 x->link[i].mass2->forceZ -= FZ;
             }
-
             for (i=0; i<x->nb_NLlink; i++)
             // compute forces generated by NLlinks (spring / dashpot)
             {
@@ -179,6 +182,7 @@ t_int *pmpd3d_tilde_perform(t_int *w)
                 LY = x->NLlink[i].mass2->posY - x->NLlink[i].mass1->posY;
                 LZ = x->NLlink[i].mass2->posZ - x->NLlink[i].mass1->posZ;
                 L = sqrt(LY*LY + LX*LX + LZ*LZ);
+
                 if ((L < x->NLlink[i].Lmax) && (L > x->NLlink[i].Lmin)) {
                     deltaL = L - x->NLlink[i].L0;
                     F = x->NLlink[i].K * pow(fabs(deltaL), x->NLlink[i].Pow);
@@ -203,29 +207,72 @@ t_int *pmpd3d_tilde_perform(t_int *w)
                 }
                 else x->NLlink[i].L = L; // on la sauve pour la prochaine itération (uniquement pour le "bang")
             }
-            for (i=0; i<x->nb_mass; i++)
-            {
-            // compute new masses position
-            // a mass does not move if M=0 (i.e : invM = 0)
-                if ( x->mass[i].D && (sqspeed = x->mass[i].speedX * x->mass[i].speedX + x->mass[i].speedY * x->mass[i].speedY + x->mass[i].speedZ * x->mass[i].speedZ) ) { // velocity damping
-                    L = sqrt(sqspeed);
-                    invL = 1 / L;
-                    F = -(L - x->mass[i].Doffset) * x->mass[i].D;
-                    x->mass[i].forceX += F * x->mass[i].speedX * invL;
-                    x->mass[i].forceY += F * x->mass[i].speedY * invL;
-                    x->mass[i].forceZ += F * x->mass[i].speedZ * invL;
-                }
-                x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
-                x->mass[i].speedY += x->mass[i].forceY * x->mass[i].invM;
-                x->mass[i].speedZ += x->mass[i].forceZ * x->mass[i].invM;
-                x->mass[i].forceX = 0;
-                x->mass[i].forceY = 0;
-                x->mass[i].forceZ = 0;
-                x->mass[i].posX += x->mass[i].speedX;
-                x->mass[i].posY += x->mass[i].speedY;
-                x->mass[i].posZ += x->mass[i].speedZ;
-            }
-        }
+            if (!x->test_minmax) 
+            { // High-performance loop, but accuracy errors can occur.
+		        for (i=0; i<x->nb_mass; i++)
+		        { // compute new masses position
+		            if ( x->mass[i].D && (sqspeed = x->mass[i].speedX * x->mass[i].speedX + x->mass[i].speedY * x->mass[i].speedY + x->mass[i].speedZ * x->mass[i].speedZ) ) { // velocity damping
+		                L = sqrt(sqspeed);
+		                invL = 1 / L;
+		                F = -(L - x->mass[i].Doffset) * x->mass[i].D;
+		                x->mass[i].forceX += F * x->mass[i].speedX * invL;
+		                x->mass[i].forceY += F * x->mass[i].speedY * invL;
+		                x->mass[i].forceZ += F * x->mass[i].speedZ * invL;
+		            }
+		            x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
+		            x->mass[i].speedY += x->mass[i].forceY * x->mass[i].invM;
+		            x->mass[i].speedZ += x->mass[i].forceZ * x->mass[i].invM;
+		            x->mass[i].forceX = 0;
+		            x->mass[i].forceY = 0;
+		            x->mass[i].forceZ = 0;
+		            x->mass[i].posX += x->mass[i].speedX;
+		            x->mass[i].posY += x->mass[i].speedY;
+		            x->mass[i].posZ += x->mass[i].speedZ;
+		        }
+		   	}
+		    else
+		    { 
+		        for (i=0; i<x->nb_mass; i++)
+		        { // compute new masses position
+				    if(x->mass[i].invM!=0) { // a mass does not move if M=0 (i.e : invM = 0)
+						if ( x->mass[i].D && (sqspeed = x->mass[i].speedX * x->mass[i].speedX + x->mass[i].speedY * x->mass[i].speedY + x->mass[i].speedZ * x->mass[i].speedZ) ) 
+						{ // velocity damping
+					        L = sqrt(sqspeed);
+					        invL = 1 / L;
+					        F = -(L - x->mass[i].Doffset) * x->mass[i].D;
+					        x->mass[i].forceX += F * x->mass[i].speedX * invL;
+					        x->mass[i].forceY += F * x->mass[i].speedY * invL;
+					        x->mass[i].forceZ += F * x->mass[i].speedZ * invL;
+					    }
+					    x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
+					    x->mass[i].speedY += x->mass[i].forceY * x->mass[i].invM;
+					    x->mass[i].speedZ += x->mass[i].forceZ * x->mass[i].invM;
+					    x->mass[i].forceX = 0;
+					    x->mass[i].forceY = 0;
+					    x->mass[i].forceZ = 0;
+					    x->mass[i].posX += x->mass[i].speedX;
+					    x->mass[i].posY += x->mass[i].speedY;
+					    x->mass[i].posZ += x->mass[i].speedZ;
+
+					    // space limitation
+					    if ((x->mass[i].posX < x->minX) || (x->mass[i].posX > x->maxX) || 
+					        (x->mass[i].posY < x->minY) || (x->mass[i].posY > x->maxY) ||
+					        (x->mass[i].posZ < x->minZ) || (x->mass[i].posZ > x->maxZ)) 
+					    {
+					        tmpX = clamp(x->mass[i].posX, x->minX, x->maxX);
+					        tmpY = clamp(x->mass[i].posY, x->minY, x->maxY);
+					        tmpZ = clamp(x->mass[i].posZ, x->minZ, x->maxZ);
+					        x->mass[i].speedX -= x->mass[i].posX - tmpX;
+					        x->mass[i].speedY -= x->mass[i].posY - tmpY;
+					        x->mass[i].speedZ -= x->mass[i].posZ - tmpZ;
+					        x->mass[i].posX = tmpX;
+					        x->mass[i].posY = tmpY;
+					        x->mass[i].posZ = tmpZ;
+					    }
+					}
+				}
+			}
+		}
 
         // compute output vector value
         for (i=0; i<x->nb_outPosX; i++) x->outlet[x->outPosX[i].nbr_outlet] += x->outPosX[i].mass->posX * x->outPosX[i].influence;
@@ -291,6 +338,7 @@ void pmpd3d_tilde_bang(t_pmpd3d_tilde *x)
     for (i=0; i<x->nb_outSpeedX; i++) logpost(x, 2, "outSpeedX:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeedX[i].nbr_outlet, x->outSpeedX[i].mass->Id, x->outSpeedX[i].influence);
     for (i=0; i<x->nb_outSpeedY; i++) logpost(x, 2, "outSpeedY:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeedY[i].nbr_outlet, x->outSpeedY[i].mass->Id, x->outSpeedY[i].influence);
     for (i=0; i<x->nb_outSpeedZ; i++) logpost(x, 2, "outSpeedZ:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeedZ[i].nbr_outlet, x->outSpeedZ[i].mass->Id, x->outSpeedZ[i].influence);
+    if(x->test_minmax) { logpost(x, 2, "minX:%f, maxX:%f, minY:%f, maxY:%f, minZ:%f, maxZ:%f", x->minX, x->maxX, x->minY, x->maxY, x->minZ, x->maxZ); }
 }
 
 inline int validate_index(t_pmpd3d_tilde *x, int idx, t_int count, const char* type)
@@ -455,6 +503,58 @@ void pmpd3d_tilde_setNLLCurrent(t_pmpd3d_tilde *x, t_symbol *s, int argc, t_atom
     x->NLlink[idx_NLlink].L0 += percent * (x->NLlink[idx_NLlink].L - x->NLlink[idx_NLlink].L0);
 }
 
+void pmpd3d_tilde_min(t_pmpd3d_tilde *x, t_float minX, t_float minY, t_float minZ)
+{
+    x->minX = minX;
+    x->minY = minY;
+    x->minZ = minZ;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_max(t_pmpd3d_tilde *x, t_float maxX, t_float maxY, t_float maxZ)
+{
+    x->maxX = maxX;
+    x->maxY = maxY;
+    x->maxZ = maxZ;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_minX(t_pmpd3d_tilde *x, t_float min)
+{
+    x->minX = min;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_maxX(t_pmpd3d_tilde *x, t_float max)
+{
+    x->maxX = max;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_minY(t_pmpd3d_tilde *x, t_float min)
+{
+    x->minY = min;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_maxY(t_pmpd3d_tilde *x, t_float max)
+{
+    x->maxY = max;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_minZ(t_pmpd3d_tilde *x, t_float min)
+{
+    x->minZ = min;
+    x->test_minmax = 1;
+}
+
+void pmpd3d_tilde_maxZ(t_pmpd3d_tilde *x, t_float max)
+{
+    x->maxZ = max;
+    x->test_minmax = 1;
+}
+
 inline int validate_count(t_pmpd3d_tilde *x, t_int count, t_int count_max, const char* type)
 {
     if (count == count_max) {
@@ -688,6 +788,12 @@ void pmpd3d_tilde_reset(t_pmpd3d_tilde *x)
     x->nb_link      = 0;
     x->nb_NLlink    = 0;
     x->nb_mass      = 0;
+    x->minX         = -1000000;
+    x->maxX         = 1000000;
+    x->minY         = -1000000;
+    x->maxY         = 1000000;
+    x->minZ         = -1000000;
+    x->maxZ         = 1000000;
     x->nb_inPosX    = 0;
     x->nb_inPosY    = 0;
     x->nb_inPosZ    = 0;
@@ -701,6 +807,7 @@ void pmpd3d_tilde_reset(t_pmpd3d_tilde *x)
     x->nb_outPosX   = 0;
     x->nb_outPosY   = 0;
     x->nb_outPosZ   = 0;
+    x->test_minmax = 0;
 }
 
 void pmpd3d_tilde_free(t_pmpd3d_tilde *x)
@@ -741,14 +848,18 @@ void *pmpd3d_tilde_new(t_symbol *s, int argc, t_atom *argv)
 
     // check for flags (currently need to be positioned first)
     while (argc && argv->a_type == A_SYMBOL) {
-        if (atom_getsymbol(argv) == gensym("-m"))
+        t_symbol *flag = atom_getsymbol(argv);
+        if (flag == gensym("-m")) {
             if(g_signal_setmultiout)
                 x->multichannel = 1;
             else
                 pd_error(x, "[pmpd3d~]: no multichannel support in Pd %i.%i-%i, ignoring '-m' flag", maj, min, bug);
-        else
-            pd_error(x, "[pmpd~]: invalid argument");
-        argc--, argv++;
+            argc--, argv++;
+        }
+        else {
+            pd_error(x, "[pmpd3d~]: invalid argument %s", flag->s_name);
+            argc--, argv++;
+        }
     }
 
     x->nb_inlet = max(1, atom_getintarg(0, argc, argv));
@@ -764,9 +875,9 @@ void *pmpd3d_tilde_new(t_symbol *s, int argc, t_atom *argv)
     x->inlet_vector = (t_sample **)getbytes(x->nb_inlet * sizeof(t_sample *));
     x->outlet_vector = (t_sample **)getbytes(x->nb_outlet * sizeof(t_sample *));
 
-    x->mass   = (struct _mass *)getbytes(x->nb_max_mass * sizeof(struct _link));
+    x->mass   = (struct _mass *)getbytes(x->nb_max_mass * sizeof(struct _mass));
     x->link   = (struct _link *)getbytes(x->nb_max_link * sizeof(struct _link));
-    x->NLlink = (struct _NLlink *)getbytes(x->nb_max_link * sizeof(struct _link));
+    x->NLlink = (struct _NLlink *)getbytes(x->nb_max_link * sizeof(struct _NLlink));
 
     x->inPosX    = (struct _input *)getbytes(x->nb_max_in * sizeof(struct _input));
     x->inPosY    = (struct _input *)getbytes(x->nb_max_in * sizeof(struct _input));
@@ -863,6 +974,12 @@ PMPD_EXPORT void pmpd3d_tilde_setup(void)
     class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_setNLLMin, gensym("setNLLMin"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_setNLLMax, gensym("setNLLMax"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_setNLLCurrent, gensym("setNLLCurrent"), A_GIMME, 0);
+    class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_minX, gensym("minX"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_maxX, gensym("maxX"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_minY, gensym("minY"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_maxY, gensym("maxY"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_minZ, gensym("minZ"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_maxZ, gensym("maxZ"), A_DEFFLOAT, 0);
 
     class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_reset, gensym("reset"), 0);
     class_addmethod(pmpd3d_tilde_class, (t_method)pmpd3d_tilde_dsp, gensym("dsp"), A_CANT, 0);
