@@ -31,6 +31,8 @@
 #include "pmpd_version.h"
 
 #define max(a,b) ( ((a) > (b)) ? (a) : (b) )
+#define min(a,b) ( ((a) < (b)) ? (a) : (b) )
+#define clamp(x,lo,hi) ( min(hi, max(lo, x)) )
 
 #define NB_MAX_LINK_DEFAULT 10000
 #define NB_MAX_MASS_DEFAULT 10000
@@ -95,6 +97,8 @@ typedef struct _pmpd_tilde {
     t_int nb_inPos, nb_inForce, nb_outPos, nb_outSpeed;
     t_sample f; // used for signal inlet
     t_int nb_loop; // to be able not to compute everything a each iteration
+    t_float minX, maxX;
+    t_int test_minmax;
 } t_pmpd_tilde;
 
 t_int *pmpd_tilde_perform(t_int *w)
@@ -103,7 +107,7 @@ t_int *pmpd_tilde_perform(t_int *w)
     t_pmpd_tilde *x = (t_pmpd_tilde *)(w[1]);
     t_int n = w[2]; // sample count from sp[0]->s_n
 
-    t_float F,L;
+    t_float F,L,tmpX;
     t_int i, si, loop;
 
     for (si=0; si<n; si++)
@@ -135,6 +139,7 @@ t_int *pmpd_tilde_perform(t_int *w)
             {
                 L=x->NLlink[i].mass1->posX - x->NLlink[i].mass2->posX - x->NLlink[i].L0;
                 x->NLlink[i].L = L;
+
                 if ((L < x->NLlink[i].Lmax) && (L > x->NLlink[i].Lmin))
                 {
                     F = x->NLlink[i].K * pow((fabs(L)), x->NLlink[i].Pow);
@@ -148,16 +153,34 @@ t_int *pmpd_tilde_perform(t_int *w)
                     x->NLlink[i].mass2->forceX += F;
                 }
             }
-            for (i=0; i<x->nb_mass; i++)
-            {
-            // compute new masses position
-            // a mass does not move if M=0 (i.e : invM = 0)
-                x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
-                x->mass[i].forceX = 0; //random_bang_pmpd_tilde(x) * 1e-25;
-                    // only used for denormal problem
-                    // -ffast-math -O6 does not solve the problem
-                x->mass[i].posX += x->mass[i].speedX;
+            if (!x->test_minmax) { // High-performance loop, but accuracy errors can occur.
+		        for (i=0; i<x->nb_mass; i++)
+		        { // compute new masses position
+			        x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
+			        x->mass[i].forceX = 0; 
+			        x->mass[i].posX += x->mass[i].speedX;     
+		        }
             }
+            else  // slower loop but more robust computation
+            { // lot's of test for better accuracy
+                for (i=0; i<x->nb_mass; i++)
+		        { // compute new masses position
+		        	if(x->mass[i].invM!=0) 
+		        	{ // a mass does not move if M=0 (i.e : invM = 0)
+				        x->mass[i].speedX += x->mass[i].forceX * x->mass[i].invM;
+				        x->mass[i].forceX = 0; 
+				        x->mass[i].posX += x->mass[i].speedX;
+
+				        // space limitation
+				         if ((x->mass[i].posX < x->minX) || (x->mass[i].posX > x->maxX)) 
+				         {
+				            tmpX = clamp(x->mass[i].posX, x->minX, x->maxX);
+				            x->mass[i].speedX -= x->mass[i].posX - tmpX;
+				            x->mass[i].posX = tmpX;
+				         }     
+		             }
+		        }
+			}
         }
 
         // compute output vector value
@@ -207,11 +230,12 @@ void pmpd_tilde_bang(t_pmpd_tilde *x)
     t_int i;
     for (i=0; i<x->nb_mass; i++)     logpost(x, 2, "mass:%ld, M:%f, pos:%f", i, x->mass[i].invM > 0 ? 1/x->mass[i].invM : 0, x->mass[i].posX);
     for (i=0; i<x->nb_link; i++)     logpost(x, 2, "link:%ld, mass1:%ld, mass2:%ld, K:%f, D:%f", i, x->link[i].mass1->Id, x->link[i].mass2->Id, x->link[i].K, x->link[i].D);
-    for (i=0; i<x->nb_NLlink; i++)   logpost(x, 2, "NLlink:%ld, mass1:%ld, mass2:%ld, K:%f, D:%f, L0:%f, Lmin:%f, Lmax:%f, Pow:%f", i, x->NLlink[i].mass1->Id, x->NLlink[i].mass2->Id, x->NLlink[i].K, x->NLlink[i].D, x->NLlink[i].L0, x->NLlink[i].Lmin, x->NLlink[i].Lmax, x->NLlink[i].Pow);
+    for (i=0; i<x->nb_NLlink; i++)   logpost(x, 2, "NLlink:%ld, mass1:%ld, mass2:%ld, K:%f, D:%f, L0:%f, L:%f, Lmin:%f, Lmax:%f, Pow:%f", i, x->NLlink[i].mass1->Id, x->NLlink[i].mass2->Id, x->NLlink[i].K, x->NLlink[i].D, x->NLlink[i].L0, x->NLlink[i].L, x->NLlink[i].Lmin, x->NLlink[i].Lmax, x->NLlink[i].Pow);
     for (i=0; i<x->nb_inPos; i++)    logpost(x, 2, "inPos:%ld, Inlet:%ld, Mass:%ld, Amplitude:%f", i, x->inPos[i].nbr_inlet, x->inPos[i].mass->Id, x->inPos[i].influence);
     for (i=0; i<x->nb_inForce; i++)  logpost(x, 2, "inForce:%ld, Inlet:%ld, Mass:%ld, Amplitude:%f", i, x->inForce[i].nbr_inlet, x->inForce[i].mass->Id, x->inForce[i].influence);
     for (i=0; i<x->nb_outPos; i++)   logpost(x, 2, "outPos:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outPos[i].nbr_outlet, x->outPos[i].mass->Id, x->outPos[i].influence);
     for (i=0; i<x->nb_outSpeed; i++) logpost(x, 2, "outSpeed:%ld, Outlet:%ld, Mass:%ld, Amplitude:%f", i, x->outSpeed[i].nbr_outlet, x->outSpeed[i].mass->Id, x->outSpeed[i].influence);
+    if(x->test_minmax) { logpost(x, 2, "minX:%f, maxX:%f", x->minX, x->maxX); }
 }
 
 inline int validate_index(t_pmpd_tilde *x, int idx, t_int count, const char* type)
@@ -324,6 +348,19 @@ void pmpd_tilde_setNLLCurrent(t_pmpd_tilde *x, t_symbol *s, int argc, t_atom *ar
     x->NLlink[idx_NLlink].L0 += percent * (x->NLlink[idx_NLlink].L - x->NLlink[idx_NLlink].L0);
 }
 
+
+void pmpd_tilde_minX(t_pmpd_tilde *x, t_float min)
+{
+    x->minX = min;
+    x->test_minmax = 1;
+}
+
+void pmpd_tilde_maxX(t_pmpd_tilde *x, t_float max)
+{
+    x->maxX = max;
+    x->test_minmax = 1;
+}
+
 inline int validate_count(t_pmpd_tilde *x, t_int count, t_int count_max, const char* type)
 {
     if (count == count_max)
@@ -438,10 +475,13 @@ void pmpd_tilde_reset(t_pmpd_tilde *x)
     x->nb_link     = 0;
     x->nb_NLlink   = 0;
     x->nb_mass     = 0;
+    x->minX         = -1000000;
+    x->maxX         = 1000000;
     x->nb_inPos    = 0;
     x->nb_inForce  = 0;
     x->nb_outSpeed = 0;
     x->nb_outPos   = 0;
+    x->test_minmax = 0;
 }
 
 void pmpd_tilde_free(t_pmpd_tilde *x) {
@@ -472,14 +512,18 @@ void *pmpd_tilde_new(t_symbol *s, int argc, t_atom *argv)
 
     // check for flags (currently need to be positioned first)
     while (argc && argv->a_type == A_SYMBOL) {
-        if (atom_getsymbol(argv) == gensym("-m")) {
+        t_symbol *flag = atom_getsymbol(argv);
+        if (flag == gensym("-m")) {
             if(g_signal_setmultiout)
                 x->multichannel = 1;
             else
                 pd_error(x, "[pmpd~]: no multichannel support in Pd %i.%i-%i, ignoring '-m' flag", maj, min, bug);
-        } else
-            pd_error(x, "[pmpd~]: invalid argument");
-        argc--, argv++;
+            argc--, argv++;
+        }
+        else {
+            pd_error(x, "[pmpd~]: invalid argument %s", flag->s_name);
+            argc--, argv++;
+        }
     }
 
     x->nb_inlet = max(1, atom_getintarg(0, argc, argv));
@@ -495,14 +539,15 @@ void *pmpd_tilde_new(t_symbol *s, int argc, t_atom *argv)
     x->inlet_vector = (t_sample **)getbytes(x->nb_inlet * sizeof(t_sample *));
     x->outlet_vector = (t_sample **)getbytes(x->nb_outlet * sizeof(t_sample *));
 
-    x->mass     = (struct _mass *)getbytes(x->nb_max_mass * sizeof(struct _link));
+    x->mass     = (struct _mass *)getbytes(x->nb_max_mass * sizeof(struct _mass));
     x->link     = (struct _link *)getbytes(x->nb_max_link * sizeof(struct _link));
-    x->NLlink   = (struct _NLlink *)getbytes(x->nb_max_link * sizeof(struct _link));
+    x->NLlink   = (struct _NLlink *)getbytes(x->nb_max_link * sizeof(struct _NLlink));
 
     x->inPos    = (struct _input *)getbytes(x->nb_max_in * sizeof(struct _input));
     x->inForce  = (struct _input *)getbytes(x->nb_max_in * sizeof(struct _input));
     x->outPos   = (struct _output *)getbytes(x->nb_max_out * sizeof(struct _output));
     x->outSpeed = (struct _output *)getbytes(x->nb_max_out * sizeof(struct _output));
+    x->test_minmax = 0;
 
     outlet_new(&x->x_obj, &s_signal);
     // add more channels if multichannel not set
@@ -575,6 +620,10 @@ PMPD_EXPORT void pmpd_tilde_setup(void)
     class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_setNLLMin, gensym("setNLLMin"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_setNLLMax, gensym("setNLLMax"), A_DEFFLOAT, A_DEFFLOAT, 0);
     class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_setNLLCurrent, gensym("setNLLCurrent"), A_GIMME, 0);
+    class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_minX, gensym("min"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_minX, gensym("minX"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_maxX, gensym("max"), A_DEFFLOAT, 0);
+    class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_maxX, gensym("maxX"), A_DEFFLOAT, 0);
 
     class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_reset, gensym("reset"), 0);
     class_addmethod(pmpd_tilde_class, (t_method)pmpd_tilde_dsp, gensym("dsp"), A_CANT, 0);
